#! /usr/bin/env python3

# This program is placed in the public domain.

from pathlib import Path
import sys
import xlsxwriter

if len(sys.argv) < 2:
    print(f"""Aufruf: {sys.argv[0]} DATEINAME
      DATEINAME: Name der txt-Datei
      Diese kann aus der PDF-Datei erzeugt werden mit
          pdftotext -layout -fixed 1.5 KBV-Datenpakete.pdf
""")
    sys.exit(1)
          
PATH = Path(sys.argv[1]).expanduser()

# empty columns found:
# empty = 0-33, 40-43, 50-51, 58-59, 66-67, 74-75, 82-84, 91-92, 99-100, 107-108, 115-116, 123-124,
#         131-133, 140-141, 148-149, 156-157, 164-165, 172-173, 181, 188-189, 197, 205, 213,
#         221-222, 230, 238, 246-247, 256-257, 266, 275-276, 285-286, 295-296, 305, 314-315,
#         324-325, 334-335, 344-345, 354, 363-364, 373-374, 383-384, 393, 402-403, 412-413,
#         422-423, 432, 441-442, 451-452, 461-462, 471, 480-481, 490, 491-499}

starts = [34, 44, 52, 60, 68, 76, 85, 93, 101, 109, 117, 125, 134, 142, 150, 158, 166, 174, 182,
        190, 198, 206, 214, 223, 231, 239, 248, 258, 267, 277, 287, 297, 306, 316, 326, 336, 346,
        355, 365, 375, 385, 394, 404, 414, 424, 433, 443, 453, 463, 472, 482]
ends =   [40, 50, 58, 66, 74, 82, 91, 99, 107, 115, 123, 131, 140, 148, 156, 164, 172, 181, 188,
        197, 205, 213, 221, 230, 238, 246, 256, 266, 275, 285, 295, 305, 314, 324, 334, 344, 354,
        363, 373, 383, 393, 402, 412, 422, 432, 441, 451, 461, 471, 480, 490]

result = []

fields = list(zip(starts, ends))
spaces = list(zip(ends[:-1], starts[1:]))

for number, line in enumerate(PATH.read_text().splitlines()):
    if number < 26:
        # skip broken header lines
        continue
    line.removeprefix("\x0c")  # remove page breaks
    l = len(line)
    
    gaps = [line[from_:to_] for from_, to_ in spaces]
    gaps = [g for g in gaps if g.strip()]
    if gaps:
        print(f"non-empty gap in line {number}: {gaps}")
    s_values = [line[from_:to_].strip() or "0" for from_, to_ in fields]
    problems = [v for v in s_values if " " in v]
    if problems:
        print(f"blank in {problems} in line {number}")
        continue
    values = [int(v) for v in s_values[1:]]
    if not [v for v in values if v]:
        # skip empty lines
        continue
    result.append((s_values[0], values))

wb = xlsxwriter.Workbook(PATH.with_suffix(".xlsx"))
heading = wb.add_format(dict(bold=True, align="center"))
col_width = 10
ws = wb.add_worksheet("KBV")
line = 0
ws.write(line, 0, "Diagnose", heading)
col = 1
for kind in "", "no ":
    for year in range(2016, 2023):
        for quarter in range(1, 5):
            if year >=2022 and quarter > 1:
                break
            ws.set_column(col, col, col_width)
            ws.write(line, col, f"{kind}{year}/{quarter}", heading)
            col += 1
line += 1

for code, values in result:
    ws.write(line, 0, code)
    for col, value in enumerate(values):
        ws.write(line, col + 1, value)
    line += 1
ws.freeze_panes(1, 1)
wb.close()
